#include <windows.h>

// Индентификатор таймера
#define IDT_TIMER1 1001

// Объявление процедур для окна
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM);

// Основные функции для работы
class Application {

protected:

    WNDCLASS wc;
    HWND     hWnd;
    HDC      hDC;
    HGLRC    hRC;
    MSG      msg;
    BOOL     bQuit = FALSE;
    int      width, height;

public:

    virtual void InitHook() { }

    // Запуск кода
    int start(HINSTANCE hInstance, LPCSTR title, int _width, int _height) {

        // Регистрация класса
        wc.style           = CS_OWNDC;
        wc.lpfnWndProc     = WndProc;
        wc.cbClsExtra      = 0;
        wc.cbWndExtra      = 0;
        wc.hInstance       = hInstance;
        wc.hIcon           = LoadIcon (NULL, IDI_APPLICATION);
        wc.hCursor         = LoadCursor (NULL, IDC_ARROW);
        wc.hbrBackground   = (HBRUSH) GetStockObject (BLACK_BRUSH);
        wc.lpszMenuName    = NULL;
        wc.lpszClassName   = "APPClass";
        RegisterClass (&wc);

        // Установка размера
        width  = _width;
        height = _height;

        // Создать основное окно
        hWnd = CreateWindow (
            "APPClass",
            title, WS_CAPTION | WS_POPUPWINDOW | WS_VISIBLE,
            0, 0, width, height, NULL, NULL, hInstance, NULL);

        // Подрубка OpenGL
        EnableOpenGL (hWnd, &hDC, &hRC);

        // Вызвать функцию потомка
        InitHook();

        // Создать таймер
        SetTimer(hWnd, IDT_TIMER1, 20, NULL);

        // Основной программный цикл
        while (!bQuit)
        {
            // Проверить новые сообщения
            if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE)) {

                // Выход
                if (msg.message == WM_QUIT) {
                    bQuit = TRUE;
                } else {
                    TranslateMessage (&msg);
                    DispatchMessage (&msg);
                }
            }
        }

        // Удалить таймер
        KillTimer(hWnd, IDT_TIMER1);

        // Остановка OpenGL
        DisableOpenGL (hWnd, hDC, hRC);

        // Выключить окно
        DestroyWindow (hWnd);

        return msg.wParam;
    }

    // Обработчик события окна
    LRESULT CALLBACK EventWnd(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {

        switch (message)
        {
            // Создание окна
            case WM_CREATE:
                return 0;

            // Закрытие окна
            case WM_CLOSE:

                PostQuitMessage (0);
                return 0;

            // Удаление окна из памяти
            case WM_DESTROY:
                return 0;

            // Нажатие на клавишу
            case WM_KEYDOWN:

                switch (wParam)
                {
                    case VK_ESCAPE:

                        PostQuitMessage(0);
                        return 0;
                }
                return 0;

            // Таймер
            case WM_TIMER:
                return 0;

            default:
                return DefWindowProc (hWnd, message, wParam, lParam);
        }
    }

    // Включение OpenGL
    void EnableOpenGL (HWND hWnd, HDC *hDC, HGLRC *hRC) {

        PIXELFORMATDESCRIPTOR pfd;
        int iFormat;

        /* get the device context (DC) */
        *hDC = GetDC (hWnd);

        /* set the pixel format for the DC */
        ZeroMemory (&pfd, sizeof (pfd));
        pfd.nSize = sizeof (pfd);
        pfd.nVersion = 1;
        pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
        pfd.iPixelType = PFD_TYPE_RGBA;
        pfd.cColorBits = 24;
        pfd.cDepthBits = 16;
        pfd.iLayerType = PFD_MAIN_PLANE;
        iFormat = ChoosePixelFormat (*hDC, &pfd);
        SetPixelFormat (*hDC, iFormat, &pfd);

        /* create and enable the render context (RC) */
        *hRC = wglCreateContext( *hDC );
        wglMakeCurrent( *hDC, *hRC );
    }

    // Отключение OpenGL
    void DisableOpenGL (HWND hWnd, HDC hDC, HGLRC hRC) {

        wglMakeCurrent (NULL, NULL);
        wglDeleteContext (hRC);
        ReleaseDC (hWnd, hDC);
    }
};