#include <math.h>
#include <gl/gl.h>
#include <gl/GLU.h>

#include "app.cpp"

// Главное окно-обработчик
class AppWindow : public Application {
protected:

    float   theta;
    GLuint  texture_id;

public:

    // Наследованная виртуальная функция
    void InitHook() {

        // Включение возможностей
        glEnable(GL_DEPTH_TEST);                // Включить Z-буфер
        glEnable(GL_TEXTURE_2D);                // Включить текстурирование

        int w = 512, h = 512;
        GLuint bgra[w*h];

        // Создать тестовую текстуру
        for (int i = 0; i < h; i++) {
        for (int j = 0; j < w; j++) {
            bgra[i*h + j] = i ^ j;
        } }

        // Текстура создается в памяти, назначается GLuint
        glGenTextures(1, (GLuint *) & texture_id);

        // Выбираем эту текстуру, работаем с 2D: x, y.
        glBindTexture(GL_TEXTURE_2D, texture_id);

        // Интерполяция для приближенных и удаленных объектов
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

        // Повторять текстуры
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

        // https://www.khronos.org/registry/OpenGL-Refpages/gl4/html/glTexImage2D.xhtml
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, bgra);
    }

    // Отслеживаемое событие
    LRESULT CALLBACK EventProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {

        // Обработка таймера
        switch (message) {

            case WM_TIMER:

                switch (wParam)
                {
                    case IDT_TIMER1:

                        // Построение перспективной проекции
                        glMatrixMode(GL_PROJECTION);
                        glLoadIdentity();
                        gluPerspective(50, 1, 1, 1000);

                        // Сформировать угол обзора
                        glMatrixMode(GL_MODELVIEW);
                        glLoadIdentity();
                        gluLookAt(0.0, 0.0, 2.0, // Глаз
                                  0.0, 0.0, 0.0, // Куда смотрим
                                  0.0, 1.0, 0.0); // Направление верха

                        // Анимация начинается тут
                        glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
                        glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

                        // Нарисовать треугольник
                        glRotatef (theta, 0.0f, 0.0f, 1.0f);
                        glBegin (GL_TRIANGLES);
                        glBindTexture(GL_TEXTURE_2D, texture_id);
                        glTexCoord2f(0.0f, 0.0f); glVertex3f (-1.0f,  1.0f, -1.0f);
                        glTexCoord2f(1.0f, 0.0f); glVertex3f ( 1.0f,  1.0f, -1.0f);
                        glTexCoord2f(1.0f, 1.0f); glVertex3f ( 1.0f, -1.0f, -1.0f);
                        glEnd ();

                        // Обмен буферами
                        SwapBuffers (hDC);

                        theta += 1.0f;
                        return 0;
                }
        }

        return EventWnd(hWnd, message, wParam, lParam);
    }
};
